@@ -1,21 +1,20 @@
 //! Arithmetic coding library
 
-#![deny(missing_docs, clippy::all, missing_debug_implementations, clippy::cargo)]
+#![deny(
+    missing_docs,
+    clippy::all,
+    missing_debug_implementations,
+    clippy::cargo
+)]
 #![warn(clippy::pedantic)]
-#![feature(int_log)]
-#![feature(associated_type_defaults)]
 
-mod model;
-pub use model::Model;
-
-mod encoder;
-pub use encoder::Encoder;
+pub use arithmetic_coding_core::{fixed_length, BitStore, Model};
 
 mod decoder;
-pub use decoder::Decoder;
+mod encoder;
 
-mod bitstore;
-pub use bitstore::BitStore;
+pub use decoder::Decoder;
+pub use encoder::Encoder;
 
 /// Errors that can occur during encoding/decoding
 #[derive(Debug, thiserror::Error)]
