@@ -1,15 +1,15 @@
 use arithmetic_coding::{Decoder, Encoder, Model};
 use bitstream_io::{BigEndian, BitReader, BitWrite, BitWriter};
 
-pub fn round_trip<M>(model: M, input: Vec<M::Symbol>)
+pub fn round_trip<M>(model: M, input: &[M::Symbol])
 where
     M: Model + Clone,
     M::Symbol: PartialEq + std::fmt::Debug + Clone,
 {
-    let buffer = encode(model.clone(), input.clone());
+    let buffer = encode(model.clone(), input.to_owned());
     let output = decode(model, &buffer);
 
-    assert_eq!(input, output);
+    assert_eq!(input, output.as_slice());
 }
 
 fn encode<M>(model: M, input: Vec<M::Symbol>) -> Vec<u8>
@@ -17,9 +17,9 @@ where
     M: Model,
 {
     let mut bitwriter = BitWriter::endian(Vec::new(), BigEndian);
-    let mut encoder = Encoder::<M>::new(model);
+    let mut encoder = Encoder::new(model, &mut bitwriter);
 
-    encoder.encode_all(input, &mut bitwriter).unwrap();
+    encoder.encode_all(input).unwrap();
     bitwriter.byte_align().unwrap();
 
     bitwriter.into_writer()
@@ -30,11 +30,7 @@ where
     M: Model,
 {
     let bitreader = BitReader::endian(buffer, BigEndian);
-    let mut decoder = Decoder::new(model, bitreader).unwrap();
-    let mut output = Vec::new();
+    let mut decoder = Decoder::new(model, bitreader);
 
-    while let Some(symbol) = decoder.decode_symbol().unwrap() {
-        output.push(symbol);
-    }
-    output
+    decoder.decode_all().map(Result::unwrap).collect()
 }
